---

title: HierarchicalFilter

---

# HierarchicalFilter

Renders a hierarchical filter recursively, emitting the needed events when clicked.

## Props

| Name                              | Description                                                                | Type                                           | Default               |
| --------------------------------- | -------------------------------------------------------------------------- | ---------------------------------------------- | --------------------- |
| <code>filter</code>               | The filter data to render.                                                 | <code>HierarchicalFilterModel</code>           | <code></code>         |
| <code>childrenAnimation</code>    | The animation component to use for the children filters.                   | <code>AnimationProp</code>                     | <code></code>         |
| <code>clickEvents</code>          | Additional events, with their payload, to emit when the filter is clicked. | <code>Partial<XEventsTypes></code>             | <code></code>         |
| <code>cssClasses</code>           | Inheritance CSS classes.                                                   | <code>(string \| Dictionary<boolean>)[]</code> | <code>() => []</code> |
| <code>childrenFiltersClass</code> | Class inherited by content element.                                        | <code>string</code>                            | <code></code>         |
| <code>filterItemClass</code>      | Class inherited by content element.                                        | <code>string</code>                            | <code></code>         |

## Slots

| Name                 | Description                              | Bindings<br />(name - type - description) |
| -------------------- | ---------------------------------------- | ----------------------------------------- |
| <code>default</code> | The content to render inside the button. | <br /><br /><br />                        |
| <code>label</code>   | The content to render inside the button. |                                           |

## Events

A list of events that the component will emit:

- [`UserClickedAFilter`](https://github.com/empathyco/x/blob/main/packages/x-components/src/wiring/events.types.ts):
  the event is emitted after the user clicks the button, using the `filter` prop as its payload.
- [`UserClickedAHierarchicalFilter`](https://github.com/empathyco/x/blob/main/packages/x-components/src/wiring/events.types.ts):
  the event is emitted after the user clicks the button, using the `filter` prop as its payload.
  filter.

## See it in action

This component renders a button, which on clicked emits the `UserClickedAFilter` and
`UserClickedAHierarchicalFilter` events. By default it renders the filter label as the button text.
If the provided filter has children filters, this component will render them recursively. Changing
the slot content will change it for all of the children.

The `filter` prop is required. The `clickEvents` prop is optional and allows configuring the events
to emit on click.

```vue
<template>
  <HierarchicalFilter :filter="filter" />
</template>

<script>
import { HierarchicalFilter } from '@empathyco/x-components/facets'

export default {
  name: 'HierarchicalFilterTest',
  components: {
    HierarchicalFilter,
  },
  date() {
    return {
      filter: {
        id: `categories:men`,
        modelName: 'HierarchicalFilter',
        label: `men`,
        facetId: 'categories',
        parentId: null,
        totalResults: 10,
        children: [],
        selected: false,
      },
    }
  },
}
</script>
```

### Playing with props

Configuring the events to emit when the filter is clicked.

```vue
<template>
  <HierarchicalFilter :clickEvents="{ UserClickedAHierarchicalFilter: filter }" :filter="filter" />
</template>

<script>
import { HierarchicalFilter } from '@empathyco/x-components/facets'

export default {
  name: 'HierarchicalFilterTest',
  components: {
    HierarchicalFilter,
  },
  date() {
    return {
      filter: {
        id: `categories:men`,
        modelName: 'HierarchicalFilter',
        label: `men`,
        facetId: 'categories',
        parentId: null,
        totalResults: 10,
        children: [],
        selected: false,
      },
    }
  },
}
</script>
```

### Customizing the default slot content

In this example, the child filters will also include the label and checkbox.

```vue
<template>
  <HierarchicalFilter :filter="filter" v-slot="{ filter, clickFilter, slotCSSClasses, isDisabled }">
    <label :class="slotCSSClasses">
      <input @change="clickFilter" :disabled="isDisabled" />
      {{ filter.label }}
    </label>
  </HierarchicalFilter>
</template>

<script>
import { HierarchicalFilter } from '@empathyco/x-components/facets'

export default {
  name: 'HierarchicalFilterTest',
  components: {
    HierarchicalFilter,
  },
  date() {
    return {
      filter: {
        id: `categories:men`,
        modelName: 'HierarchicalFilter',
        label: `men`,
        facetId: 'categories',
        parentId: null,
        totalResults: 10,
        children: [],
        selected: false,
      },
    }
  },
}
</script>
```

### Customizing the label slot content

```vue
<template>
  <HierarchicalFilter :filter="filter">
    <template #label :filter="filter">
      <span class="custom-class">{{ filter.label }}</span>
    </template>
  </HierarchicalFilter>
</template>

<script>
import { HierarchicalFilter } from '@empathyco/x-components/facets'

export default {
  name: 'HierarchicalFilterTest',
  components: {
    HierarchicalFilter,
  },
  date() {
    return {
      filter: {
        id: `categories:men`,
        modelName: 'HierarchicalFilter',
        label: `men`,
        facetId: 'categories',
        parentId: null,
        totalResults: 10,
        children: [],
        selected: false,
      },
    }
  },
}
</script>
```

### Customizing the content with classes

The `childrenFiltersClass` prop can be used to add classes to the inner filters lists. This is
useful to set the indent of the children filters.

The `filterItemClass` prop can be used to add classes to the filter element itself.

```vue
<template>
  <HierarchicalFilter
    :filter="filter"
    childrenFiltersClass="x-custom-class"
    filterItemClass="x-custom-filter"
  />
</template>

<script>
import { HierarchicalFilter } from '@empathyco/x-components/facets'

export default {
  name: 'HierarchicalFilterTest',
  components: {
    HierarchicalFilter,
  },
  date() {
    return {
      filter: {
        id: `categories:men`,
        modelName: 'HierarchicalFilter',
        label: `men`,
        facetId: 'categories',
        parentId: null,
        totalResults: 10,
        children: [],
        selected: false,
      },
    }
  },
}
</script>
```
