---

title: RelatedTags

---

# RelatedTags

This component renders a set of [`RelatedTag`](./x-components.related-tag) components by
default to select from after a query is performed to fine-tune search.
For example, if you are searching for _lego_, different related tags could be _city_,
friends*, or *harry potter*, refining the search with *lego city*, *lego friends*,
or *lego harry potter\*.

## Props

| Name                          | Description                                                         | Type                       | Default            |
| ----------------------------- | ------------------------------------------------------------------- | -------------------------- | ------------------ |
| <code>animation</code>        | Animation component that will be used to animate the suggestion.    | <code>AnimationProp</code> | <code>'ul'</code>  |
| <code>maxItemsToRender</code> | Number of related tags to be rendered.                              | <code>number</code>        | <code></code>      |
| <code>highlightCurated</code> | Flag to indicate if the curated tags should be displayed different. | <code>boolean</code>       | <code>false</code> |
| <code>itemClass</code>        | Class inherited by content element.                                 | <code>string</code>        | <code></code>      |

## Slots

| Name                             | Description                                                  | Bindings<br />(name - type - description)                                                                                                                                                                                     |
| -------------------------------- | ------------------------------------------------------------ | ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| <code>related-tag</code>         | Custom content that replaces the RelatedTag component.       | **relatedTag** <code>RelatedTag</code> - Related tag data.<br />**highlightCurated** <code>boolean</code> - True if the curated RTs should be displayed.                                                                      |
| <code>related-tag-content</code> | Custom content that replaces the RelatedTag default content. | **relatedTag** <code>RelatedTag</code> - Related tag data.<br />**isSelected** <code>boolean</code> - Related tag status.<br />**shouldHighlightCurated** <code>boolean</code> - True if the curated RTs should be displayed. |

## See it in action

<!-- prettier-ignore-start -->
:::warning Backend microservice required
To use this component, the QuerySignals microservice must be implemented.
:::
<!-- prettier-ignore-end -->

This example shows how related tags can be rendered without any additional effects.

_Search for a fashion term like "sandal" or "lipstick"._

```vue live
<template>
  <div>
    <SearchInput />
    <RelatedTags />
  </div>
</template>

<script>
import { SearchInput } from '@empathyco/x-components/search-box'
import { RelatedTags } from '@empathyco/x-components/related-tags'

export default {
  name: 'RelatedTagsDemo',
  components: {
    SearchInput,
    RelatedTags,
  },
}
</script>
```

### Play with props

In this example, the number of related tags rendered has been limited to 3. A fade and slide effect
has been added so that the related tags appear with a delay, then slide upwards and fade.

_Search for a fashion term and see the related tags with the animation effect._

```vue live
<template>
  <div>
    <SearchInput />
    <RelatedTags :animation="'StaggeredFadeAndSlide'" :maxItemsToRender="3" />
  </div>
</template>

<script>
import Vue from 'vue'
import { SearchInput } from '@empathyco/x-components/search-box'
import { RelatedTags } from '@empathyco/x-components/related-tags'
import { StaggeredFadeAndSlide } from '@empathyco/x-components'

// Registering the animation as a global component
Vue.component('StaggeredFadeAndSlide', StaggeredFadeAndSlide)
export default {
  name: 'RelatedTagsDemo',
  components: {
    SearchInput,
    RelatedTags,
  },
}
</script>
```

### Play with related-tag slot

In this example, the [`RelatedTag`](./x-components.related-tag.md) component is passed in the
`related-tag` slot (although any other component could potentially be passed).

_Search for a fashion term and see how the related tags can be rendered._

```vue live
<template>
  <div>
    <SearchInput />
    <RelatedTags #related-tag="{ relatedTag }">
      <RelatedTag :relatedTag="relatedTag" />
    </RelatedTags>
  </div>
</template>

<script>
import { SearchInput } from '@empathyco/x-components/search-box'
import { RelatedTags, RelatedTag } from '@empathyco/x-components/related-tags'

export default {
  name: 'RelatedTagsDemo',
  components: {
    SearchInput,
    RelatedTags,
    RelatedTag,
  },
}
</script>
```

### Play with related-tag-content slot

To continue the previous example, the [`RelatedTag`](./x-components.related-tag.md) component is
passed in the `related-tag-content` slot, but in addition, an HTML span tag for the text are also
passed.

_Search for a fashion term and see how the related tags are rendered._

```vue live
<template>
  <div>
    <SearchInput />
    <RelatedTags #related-tag-content="{ relatedTag }">
      <span>{{ relatedTag.tag }}</span>
    </RelatedTags>
  </div>
</template>

<script>
import { SearchInput } from '@empathyco/x-components/search-box'
import { RelatedTags } from '@empathyco/x-components/related-tags'

export default {
  name: 'RelatedTagsDemo',
  components: {
    SearchInput,
    RelatedTags,
  },
}
</script>
```

## Extending the component

Components can be combined and communicate with each other. The `RelatedTags` component can
communicate with the [`SearchInput`](../search-box/x-components.search-input.md) as follows:

_Search for a fashion term and see how the related tags can be rendered._

```vue live
<template>
  <div>
    <SearchInput />
    <RelatedTags />
    <ResultsList #result="{ item }">
      <span class="result">
        {{ item.name }}
      </span>
    </ResultsList>
  </div>
</template>

<script>
import { SearchInput } from '@empathyco/x-components/search-box'
import { RelatedTags } from '@empathyco/x-components/related-tags'
import { ResultsList } from '@empathyco/x-components/search'

export default {
  name: 'RelatedTagsDemo',
  components: {
    SearchInput,
    RelatedTags,
    ResultsList,
  },
}
</script>
```

## Customizing the related tags with classes

The `itemClass` prop can be used to add classes to the related tags.

```vue live
<template>
  <div>
    <SearchInput />
    <RelatedTags #related-tag-content="{ relatedTag }" itemClass="x-tag-outlined x-tag-auxiliary">
      <span>{{ relatedTag.tag }}</span>
    </RelatedTags>
  </div>
</template>

<script>
import { SearchInput } from '@empathyco/x-components/search-box'
import { RelatedTags } from '@empathyco/x-components/related-tags'

export default {
  name: 'RelatedTagsDemo',
  components: {
    SearchInput,
    RelatedTags,
  },
}
</script>
```
