import { isHierarchicalFacet } from '@empathyco/x-types';
import { defineComponent, ref, computed, watch } from 'vue';
import { use$x } from '../../../../composables/use-_x.js';
import { clone } from '../../../../utils/clone.js';
import { areFiltersDifferent } from '../../../../utils/filters.js';
import '../../../../utils/storage.js';
import { flatHierarchicalFilters } from '../../utils.js';
import { facetsXModule } from '../../x-module.js';

/**
 * This component allows to provide facets by prop, to add them to the state of the
 * `Facets X-Module`. These facets will be added to the `Facets X-Module` state together with
 * the facets emitted by the `Search X-Module` through the {@link SearchXEvents.FacetsChanged}
 * event.
 *
 * @public
 */
var _sfc_main = defineComponent({
    name: 'FacetsProvider',
    xModule: facetsXModule.name,
    props: {
        /**
         * An facet group identifier to distinguish the provided facets from other facets like the
         * `Search X-Module` facets.
         *
         * @public
         */
        groupId: {
            type: String,
            default: 'provided-facets',
        },
        /**
         * The facets to provide to the `Facets X-Module` state. They have to include the
         * {@link @empathyco/x-types#Filter}.
         *
         * @internal
         */
        facets: {
            type: Array,
            required: true,
        },
    },
    setup(props) {
        const $x = use$x();
        /**
         * Temporarily stores the selected filters from the {@link FacetsProvider.facets} prop.
         * This is necessary to handle the {@link FacetsXEvents.UserChangedSelectedFilters} event.
         *
         * @internal
         */
        const selectedFilters = ref(null);
        /**
         * A computed property to group the facets and the groupId. This is used by the watcher.
         *
         * @returns The FacetGroup with the facets and the group id.
         *
         * @internal
         */
        const facetsGroup = computed(() => ({ id: props.groupId, facets: props.facets }));
        /**
         * Emits the {@link FacetsXEvents.UserChangedSelectedFilters} event when the user changes
         * the selected filters.
         *
         * @param selectedFilters - The new list of selected filters.
         * @internal
         */
        $x.on('SelectedFiltersChanged', false).subscribe((selectedFiltersParams) => {
            if (selectedFilters.value === null ||
                areFiltersDifferent(selectedFilters.value, selectedFiltersParams)) {
                $x.emit('UserChangedSelectedFilters', selectedFiltersParams);
            }
            selectedFilters.value = null;
        });
        /**
         * Extracts the selected filters from the facets and stores them in the
         * {@link FacetsProvider.selectedFilters} property.
         *
         * @param facets - The facets from whom extract the selected filters.
         * @internal
         */
        const extractSelectedFilters = (facets) => {
            selectedFilters.value = facets
                .flatMap(facet => isHierarchicalFacet(facet) ? flatHierarchicalFilters(facet.filters) : facet.filters)
                .filter(filter => filter.selected);
        };
        /**
         * Emits the {@link FacetsXEvents.FacetsGroupProvided} event with the
         * {@link FacetsProvider.facetsGroup} as payload. It also extracts and saves the selected
         * filters.
         */
        watch(() => facetsGroup.value, () => {
            if (facetsGroup.value.facets) {
                const facetsGroupClone = clone(facetsGroup.value);
                $x.emit('FacetsGroupProvided', facetsGroupClone);
                extractSelectedFilters(props.facets);
            }
        }, { immediate: true });
        return () => '';
    },
});

export { _sfc_main as default };
//# sourceMappingURL=facets-provider.vue2.js.map
