import { map, objectFilter } from '@empathyco/x-utils';
import { defineComponent, computed } from 'vue';
import { useGetter } from '../../../../composables/use-getter.js';
import { AnimationProp } from '../../../../types/animation-prop.js';
import { toKebabCase } from '../../../../utils/string.js';
import { useFacets } from '../../composables/use-facets.js';
import { facetsXModule } from '../../x-module.js';

/**
 * This component renders the list of facets stored in the Facets module. Facets can be rendered
 * differently based on their purpose and this can be achieved using the exposed slots:
 * - A default and required slot.
 * - A custom slot for each facet with the facetId as its name. This allows each facet to be
 * rendered differently based on its needs.
 *
 * @public
 */
var _sfc_main = defineComponent({
    name: 'Facets',
    xModule: facetsXModule.name,
    props: {
        /** Array of facets ids used to get the selected filters for those facets. */
        facetsIds: Array,
        /** Flag to render the component even if there are no filters selected. */
        alwaysVisible: Boolean,
        /** Animation component that will be used to animate the facets. */
        animation: {
            type: AnimationProp,
            default: 'ul',
        },
        /**
         * Discriminates the facets rendered by this component. It expects a string containing facets
         * ids, comma separated. This property will include or exclude facets based on its value.
         * The default value is an empty string and the component will render all existing facets.
         *
         * @remarks
         * To behave as a `include`, simply set the facets ids, comma separated:
         * `existingFacets=[{ brand: ... }, category: { ... }, color: { ... }, price: { ... }]`
         * `renderableFacets="brand, category"`
         *
         * The component will render brand and category facets.
         *
         * On the other hand, to simulate an `exclude` behaviour and exclude a facet from being
         * rendered, append a '!' before its id:
         * `existingFacets=[{ brand: ... }, category: { ... }, color: { ... }, price: { ... }]`
         * `renderableFacets="!brand,!price"`
         *
         * The component will render category and color facets.
         */
        renderableFacets: String,
    },
    setup(props, { slots }) {
        const { selectedFiltersByFacet } = useFacets(props);
        const { facets } = useGetter('facets');
        /**
         * The facets to be rendered after filtering {@link Facets.facets} by
         * {@link Facets.renderableFacets} content.
         *
         * @returns The list of facets to be rendered.
         */
        const facetsToRender = computed(() => {
            if (!props.renderableFacets) {
                return facets.value;
            }
            else {
                const excludedRegExp = /^!/;
                const facetIds = props.renderableFacets.split(',').map(facetId => facetId.trim());
                const included = [];
                const excluded = [];
                facetIds.forEach(facetId => {
                    if (excludedRegExp.test(facetId)) {
                        excluded.push(facetId.replace(excludedRegExp, ''));
                    }
                    else {
                        included.push(facetId);
                    }
                });
                return filterFacetsToRender(included, excluded);
            }
        });
        /**
         * Transforms a dictionary of Facets including the slot name.
         *
         * @returns A dictionary of facets with the slot name.
         */
        const mappedFacets = computed(() => {
            return map(facetsToRender.value, (facetId, facet) => ({
                slotNameById: toKebabCase(facetId),
                slotNameByModelName: toKebabCase(facet.modelName),
                facet,
            }));
        });
        /**
         * Indicates if there are facets available to show.
         *
         * @returns True if there are facets available and false otherwise.
         */
        const hasFacets = computed(() => !!Object.keys(facetsToRender.value).length);
        /**
         * Filter facets dictionary retrieving those included and/or removing excluded.
         *
         * @param included - List of facets to render.
         * @param excluded - List of not renderable facets.
         *
         * @returns The filtered list of facets to render.
         */
        function filterFacetsToRender(included, excluded) {
            const hasAnyFacetIncluded = included.length > 0;
            return objectFilter(facets.value, facetKey => {
                const isIncluded = included.includes(String(facetKey));
                const isExcluded = excluded.includes(String(facetKey));
                return hasAnyFacetIncluded ? isIncluded && !isExcluded : !isExcluded;
            });
        }
        /**
         * Whether the slot is present in the template or not.
         *
         * @param name - The slot name.
         *
         * @returns True is the slot is present in the template. False otherwise.
         */
        function hasSlot(name) {
            return !!slots[name];
        }
        return {
            selectedFiltersByFacet,
            hasFacets,
            mappedFacets,
            hasSlot,
        };
    },
});

export { _sfc_main as default };
//# sourceMappingURL=facets.vue2.js.map
