import { defineComponent, computed } from 'vue';
import BaseEventButton from '../../../../components/base-event-button.vue.js';
import { useGetter } from '../../../../composables/use-getter.js';
import { isArrayEmpty } from '../../../../utils/array.js';
import { facetsXModule } from '../../x-module.js';

/**
 * This component receives a required `facet` with
 * {@link @empathyco/x-types#BooleanFilter} as prop and renders a button, which
 * on clicked emits the {@link FacetsXEvents.UserClickedAllFilter} event. By default
 * the rendered button displays a message with the facet label but this content is customizable
 * through the default slot.
 *
 * @public
 */
var _sfc_main = defineComponent({
    name: 'AllFilter',
    components: {
        BaseEventButton,
    },
    xModule: facetsXModule.name,
    props: {
        /** The facet data. */
        facet: {
            type: Object,
            required: true,
        },
    },
    setup(props) {
        /** The getter of the selectedFiltersByFacet. */
        const { selectedFiltersByFacet } = useGetter('facets');
        /**
         * The event that will be emitted when the all filter button is clicked.
         *
         * @returns The event to emit on click.
         * @internal
         */
        const clickEvent = computed(() => {
            return { UserClickedAllFilter: [props.facet.id] };
        });
        /**
         * Computed to retrieve the selected state of this component.
         *
         * @returns True if is selected false otherwise.
         */
        const isSelected = computed(() => {
            return isArrayEmpty(selectedFiltersByFacet.value?.[props.facet.id]);
        });
        /**
         * Dynamic CSS classes to apply to the component.
         *
         * @remarks This is only valid considering that in the case of HierarchicalFilters, ancestors
         * of nested selected filters are also selected.
         *
         * @returns The dynamic CSS classes to apply to the component.
         * @internal
         */
        const cssClasses = computed(() => {
            return {
                'x-selected x-all-filter--is-selected': isSelected.value,
            };
        });
        return {
            clickEvent,
            cssClasses,
            isSelected,
        };
    },
});

export { _sfc_main as default };
//# sourceMappingURL=all-filter.vue2.js.map
