import { defineComponent, ref, computed, watch } from 'vue';
import '../../../../composables/create-use-device.js';
import { use$x } from '../../../../composables/use-_x.js';
import '@vue/devtools-api';
import '../../../../plugins/devtools/timeline.devtools.js';
import '@empathyco/x-utils';
import 'rxjs/operators';
import 'rxjs';
import '../../../../plugins/devtools/colors.utils.js';
import '../../../../plugins/x-bus.js';
import '../../../../plugins/x-plugin.js';
import 'vuex';
import '@vueuse/core';
import { facetsXModule } from '../../x-module.js';

/**
 * Renders an editable number range filter. It has two input fields to handle min and max values,
 * emitting the needed events when clicked.
 *
 * It provides a default slot, with some utils bind, to customize the whole component; and two
 * named slots `apply-content` and `clear-content` to override each button content.
 *
 * If `instant` prop is true, the needed events are emitted immediately; else, apply button is
 * rendered to confirm to do it. False by default.
 *
 * If `clear` prop is true, clear button, which sets to null component min and max values, is
 * rendered. True by default.
 *
 * @public
 */
var _sfc_main = defineComponent({
    name: 'EditableNumberRangeFilter',
    xModule: facetsXModule.name,
    props: {
        /**
         * The filter data to render and edit.
         *
         * @public
         */
        filter: {
            type: Object,
            required: true,
        },
        /**
         * If `instant` prop is true, the needed events are emitted immediately; else, apply button is
         * rendered to confirm to do it. False by default.
         *
         * @public
         */
        isInstant: Boolean,
        /**
         * If `clear` prop is true, clear button, which sets to null component min and max values, is
         * rendered. True by default.
         *
         * @public
         */
        hasClearButton: {
            type: Boolean,
            default: true,
        },
        /** Class inherited by content element. */
        inputsClass: String,
        /** Class inherited by content element. */
        buttonsClass: String,
    },
    setup(props) {
        const $x = use$x();
        const rangeFilterMin = 'minimum amount';
        const rangeFilterMax = 'maximum amount';
        /**
         * Component min value.
         *
         * @internal
         */
        const min = ref(null);
        /**
         * Component max value.
         *
         * @internal
         */
        const max = ref(null);
        /**
         * Returns {@link @empathyco/x-types#RangeValue} with component min and max
         * values.
         *
         * @returns Range value object with component values.
         *
         * @internal
         */
        const range = computed(() => {
            return { min: min.value, max: max.value };
        });
        /**
         * It checks if component min and max values are valid.
         *
         * @returns True if there is any error in the component min and max values.
         *
         * @internal
         */
        const hasError = computed(() => min.value !== null && max.value !== null && min.value > max.value);
        /**
         * It checks if component min and max values are different from the ones within the filter
         * provided as property.
         *
         * @returns True if they are different.
         *
         * @internal
         */
        const areValuesDifferent = computed(() => min.value !== props.filter.range.min || max.value !== props.filter.range.max);
        /**
         * Dynamic CSS classes.
         *
         * @returns Object which contains dynamic CSS classes.
         *
         * @internal
         */
        const cssClasses = computed(() => {
            return { 'x-editable-number-range-filter--error': hasError.value };
        });
        /**
         * Checks if the range of the filter allows any value, which happens when the min is
         * null or 0 and the max is null.
         *
         * @returns True if the range of the filter allows any value.
         *
         * @internal
         */
        const isAnyRange = computed(() => !min.value && max.value === null);
        /**
         * It returns true if the property `hasClearButton` is true and there are values to clear.
         *
         * @returns True if the clear button has to be rendered.
         *
         * @internal
         */
        const renderClearButton = computed(() => props.hasClearButton && !isAnyRange.value);
        /**
         * It emits {@link FacetsXEvents.UserModifiedEditableNumberRangeFilter} event if there are no
         * errors and component `min` and `max` values are different than `filter.range` ones.
         *
         * @internal
         */
        const emitUserModifiedFilter = () => {
            if (!hasError.value && areValuesDifferent.value) {
                $x.emit('UserModifiedEditableNumberRangeFilter', {
                    ...props.filter,
                    range: range.value,
                });
            }
        };
        /**
         * It returns the number if possible or null otherwise.
         *
         * @param value - Value.
         * @returns The element value as a number if possible or null.
         *
         * @internal
         */
        const parseRangeValue = (value) => (Number.isNaN(value) ? null : value);
        /**
         * `min` setter.
         *
         * @param value - The component `min` value to be set.
         *
         * @internal
         */
        const setMin = (value) => {
            min.value = parseRangeValue(value);
        };
        /**
         * `max` setter.
         *
         * @param value - The component `max` value to be set.
         *
         * @internal
         */
        const setMax = (value) => {
            max.value = parseRangeValue(value);
        };
        /**
         * It sets component `min` and `max` values to null , and it emits the change if component is
         * working in instant mode.
         *
         * @internal
         */
        const clearValues = () => {
            min.value = null;
            max.value = null;
        };
        /**
         * It resets the min/max range values to null if the
         * {@link FacetsXEvents.UserClickedClearAllFilters} event is fired.
         *
         * @public
         */
        $x.on('UserClickedClearAllFilters', false).subscribe(clearValues);
        /**
         * It watches the filter range values passed as property and updates component range values if
         * they change.
         *
         * @param newRange - New range value.
         *
         * @internal
         */
        watch(() => props.filter.range, (newRange) => {
            min.value = newRange.min;
            max.value = newRange.max;
        }, { immediate: true, deep: true });
        /**
         * It watches range values in order to emit the event with the change if `isInstant`
         * property is true.
         *
         * @internal
         */
        watch(range, () => {
            if (props.isInstant) {
                emitUserModifiedFilter();
            }
        }, { deep: true });
        return {
            rangeFilterMin,
            rangeFilterMax,
            cssClasses,
            min,
            max,
            setMin,
            setMax,
            emitUserModifiedFilter,
            clearValues,
            hasError,
            isAnyRange,
            renderClearButton,
        };
    },
});

export { _sfc_main as default };
//# sourceMappingURL=editable-number-range-filter.vue2.js.map
