import { isHierarchicalFilter } from '@empathyco/x-types';
import { isObject } from '@empathyco/x-utils';
import { defineComponent, computed } from 'vue';
import { AnimationProp } from '../../../../types/animation-prop.js';
import { facetsXModule } from '../../x-module.js';
import FiltersList from '../lists/filters-list.vue.js';
import _sfc_main$1 from './renderless-filter.vue.js';

/**
 * Renders a hierarchical filter recursively, emitting the needed events when clicked.
 *
 * @public
 */
var _sfc_main = defineComponent({
    name: 'HierarchicalFilter',
    xModule: facetsXModule.name,
    components: { FiltersList, RenderlessFilter: _sfc_main$1 },
    props: {
        /** The filter data to render. */
        filter: {
            type: Object,
            required: true,
        },
        /** The animation component to use for the children filters. */
        childrenAnimation: AnimationProp,
        /**
         * Additional events, with their payload, to emit when the filter is clicked.
         *
         * @public
         */
        clickEvents: Object,
        /** Inheritance CSS classes. */
        cssClasses: {
            type: Array,
            default: () => [],
        },
        /** Class inherited by content element. */
        childrenFiltersClass: String,
        /** Class inherited by content element. */
        filterItemClass: String,
    },
    setup(props) {
        /**
         * The {@link XEventsTypes} to emit.
         *
         * @returns The events to emit when clicked.
         * @internal
         */
        const innerClickEvents = computed(() => ({
            UserClickedAHierarchicalFilter: props.filter,
            ...props.clickEvents,
        }));
        const isFilterPartiallySelected = (filter) => {
            const selectedChildren = filter.children?.filter(filter => filter.selected);
            const filterChildrenLength = filter.children?.length ?? 0;
            return (!!selectedChildren &&
                ((selectedChildren.length > 0 && selectedChildren.length < filterChildrenLength) ||
                    selectedChildren.some(isFilterPartiallySelected)));
        };
        /**
         * Returns if the filter is partially selected, which means having more than one child filter
         * selected, but not every of them, or having at least one child filter partially selected.
         *
         * @returns True if the filter is partially selected. False otherwise.
         * @internal
         */
        const isPartiallySelected = computed(() => isFilterPartiallySelected(props.filter));
        /**
         * Dynamic CSS classes to apply to the component.
         *
         * @returns The dynamic CSS classes to apply to the component.
         * @internal
         */
        const innerCssClasses = computed(() => [
            'x-hierarchical-filter',
            { 'x-hierarchical-filter--is-partially-selected': isPartiallySelected.value },
            { 'x-hierarchical-filter--is-selected': props.filter.selected },
            { 'x-facet-filter--is-partially-selected': isPartiallySelected.value },
            ...props.cssClasses,
        ]);
        /**
         * Gets the child filter click events, converting the payload of the events that have a
         * {@link HierarchicalFilter} as payload to the corresponding child filter.
         *
         * @param childFilter - The child filter.
         * @returns The events to emit when clicking a child.
         * @internal
         */
        const getChildFilterClickEvents = (childFilter) => {
            return Object.entries(innerClickEvents.value).reduce((clickEvents, [event, payload]) => {
                return {
                    ...clickEvents,
                    [event]: isObject(payload) &&
                        isHierarchicalFilter(payload) &&
                        childFilter !== payload
                        ? childFilter
                        : payload,
                };
            }, {});
        };
        /**
         * List of filters to render, in case that the children's array
         * is empty it will return an empty array instead of inject the ones from the parent.
         *
         * @returns A list of filters.
         * @internal
         */
        const renderedChildrenFilters = computed(() => props.filter.children ?? []);
        return {
            innerClickEvents,
            innerCssClasses,
            renderedChildrenFilters,
            getChildFilterClickEvents,
        };
    },
});

export { _sfc_main as default };
//# sourceMappingURL=hierarchical-filter.vue2.js.map
