import { defineComponent, computed } from 'vue';
import { useXBus } from '../../../../composables/use-x-bus.js';
import { facetsXModule } from '../../x-module.js';

/**
 * Renders default slot content. It binds to the default slot a
 * {@link @empathyco/x-types#BooleanFilter}, the {@link XEvent}
 * that will be emitted when clicking the content, the CSS classes and if the content should be
 * deactivated.
 *
 * @remarks The default slot expects a root element, if it receives a list of elements, it will
 * render the first element.
 *
 * @public
 */
var _sfc_main = defineComponent({
    name: 'RenderlessFilter',
    xModule: facetsXModule.name,
    inheritAttrs: false,
    props: {
        /** The filter data to render. */
        filter: {
            type: Object,
            required: true,
        },
        /** Additional events with its payload to emit when the filter is clicked. */
        clickEvents: Object,
        /** Inheritance CSS classes. */
        cssClasses: {
            type: Array,
            default: () => [],
        },
    },
    setup(props, { slots }) {
        const xBus = useXBus();
        /** Returns `true` when the filter should be disabled. */
        const isDisabled = computed(() => props.filter.totalResults === 0);
        /** CSS classes to apply to the element. */
        const innerCssClasses = computed(() => [
            'x-facet-filter',
            { 'x-selected': props.filter.selected },
            ...props.cssClasses,
        ]);
        /** The events that will be emitted when the filter is clicked. */
        const innerClickEvents = computed(() => ({
            UserClickedAFilter: props.filter,
            ...props.clickEvents,
        }));
        /** Emit filter click events to the bus. */
        function emitClickEvents() {
            Object.entries(innerClickEvents.value).forEach(([event, payload]) => {
                xBus.emit(event, payload);
            });
        }
        return () => slots.default?.({
            filter: props.filter,
            clickFilter: emitClickEvents,
            cssClasses: innerCssClasses.value,
            isDisabled: isDisabled.value,
        }) ?? '';
    },
});

export { _sfc_main as default };
//# sourceMappingURL=renderless-filter.vue.js.map
