import { isBooleanFilter } from '@empathyco/x-types';
import { defineComponent, computed, provide } from 'vue';
import { useFiltersInjection } from '../../composables/use-filters-injection.js';
import { facetsXModule } from '../../x-module.js';

/**
 * The `ExcludeFiltersWithNoResults` component filters the provided list of filters, excluding
 * those which have the `totalResults` property exactly equal to `0`. It won't remove filters with
 * no `totalResults` property.
 *
 * The new list of filters is bound to the default scoped slot. As this component does not render
 * no root element, this default slot must contain a single root node.
 *
 * @public
 */
var _sfc_main = defineComponent({
    name: 'ExcludeFiltersWithNoResults',
    xModule: facetsXModule.name,
    props: {
        /** The list of filters to be rendered as slots. */
        filters: Array,
        /**
         * This prop is used in the `HierarchicalFilter` component and only in that case. It is necessary
         * to make the `renderedFilters` to return only the filters of each level of the hierarchy.
         */
        parentId: {
            type: String,
        },
    },
    setup(props, { slots }) {
        const renderedFilters = useFiltersInjection(props);
        /**
         * Removes the filters that have exactly 0 results associated.
         *
         * @returns A sublist of the filters prop, excluding the ones with no results.
         */
        const filtersWithResults = computed(() => renderedFilters.value.filter(filter => !isBooleanFilter(filter) || filter.totalResults !== 0));
        provide('filters', filtersWithResults);
        return () => slots.default?.({ filters: filtersWithResults.value })[0] ?? '';
    },
});

export { _sfc_main as default };
//# sourceMappingURL=exclude-filters-with-no-results.vue.js.map
