import { defineComponent, computed } from 'vue';
import { AnimationProp } from '../../../../types/animation-prop.js';
import { useFiltersInjection } from '../../composables/use-filters-injection.js';
import { facetsXModule } from '../../x-module.js';

/**
 * Renders a list with a list item per each
 * {@link @empathyco/x-types#BooleanFilter} in the filters prop array.
 * Each list item has a scoped slot, passing the filter as slot prop.
 *
 * @public
 */
var _sfc_main = defineComponent({
    name: 'FiltersList',
    xModule: facetsXModule.name,
    props: {
        /**
         * The list of filters to be rendered as slots.
         *
         * @public
         */
        filters: Array,
        /**
         * This prop is used in the `HierarchicalFilter` component and only in that case. It is necessary
         * to make the `renderedFilters` to return only the filters of each level of the hierarchy.
         *
         * @public
         */
        parentId: {
            type: String,
            required: false,
        },
        /**
         * Animation component that will be used to animate the base filters.
         *
         * @public
         */
        animation: {
            type: AnimationProp,
            default: 'ul',
        },
    },
    setup(props) {
        const renderedFilters = useFiltersInjection(props);
        /**
         * It handles if the filters should be rendered.
         *
         * @returns True if there are filters.
         *
         * @public
         */
        const hasFiltersToRender = computed(() => {
            return renderedFilters?.value.length > 0;
        });
        /**
         * Checks if at least one filter is selected.
         *
         * @returns True if at least one filter is selected. False otherwise.
         * @internal
         */
        const hasSelectedFilters = computed(() => {
            return !!renderedFilters?.value.some(filter => filter.selected);
        });
        /**
         * Dynamic CSS classes for the root element of this component.
         *
         * @returns An object containing the dynamic CSS classes and a boolean indicating if they should
         * be added or not.
         */
        const cssClasses = computed(() => {
            return {
                'x-filters--has-selected-filters': hasSelectedFilters.value,
            };
        });
        return {
            hasFiltersToRender,
            cssClasses,
            renderedFilters,
        };
    },
});

export { _sfc_main as default };
//# sourceMappingURL=filters-list.vue2.js.map
