import { isBooleanFilter } from '@empathyco/x-types';
import { defineComponent, ref, computed, watch, provide } from 'vue';
import { debounce } from '../../../../utils/debounce.js';
import { normalizeString } from '../../../../utils/normalize.js';
import { useFiltersInjection } from '../../composables/use-filters-injection.js';
import { facetsXModule } from '../../x-module.js';

/**
 * Renders the filters sifted with the input query.
 *
 * @public
 */
var _sfc_main = defineComponent({
    name: 'FiltersSearch',
    xModule: facetsXModule.name,
    props: {
        /**
         * The list of filters to be rendered as slots.
         *
         * @public
         */
        filters: Array,
        /**
         * This prop is used in the `HierarchicalFilter` component and only in that case. It is necessary
         * to make the `renderedFilters` to return only the filters of each level of the hierarchy.
         *
         * @public
         */
        parentId: {
            type: String,
            required: false,
        },
        /** The debounce time for applying the filter sifting. */
        debounceInMs: {
            type: Number,
            default: 200,
        },
    },
    setup(props) {
        const renderedFilters = useFiltersInjection(props);
        const query = ref('');
        let setQueryDebounced;
        const debounceInMs = computed(() => props.debounceInMs);
        /**
         * Set the debounce function for setting the query debounced.
         *
         * @internal
         */
        const updateSetQueryDebounced = () => {
            setQueryDebounced = debounce(queryDebounced => {
                query.value = queryDebounced;
            }, props.debounceInMs);
        };
        watch(debounceInMs, updateSetQueryDebounced, { immediate: true });
        /**
         * Sift the array of filters which matches with the query.
         *
         * @returns Array of sifted filters.
         * @internal
         */
        const siftedFilters = computed(() => {
            const normalizedQuery = normalizeString(query.value);
            return renderedFilters.value.filter(filter => isBooleanFilter(filter) && normalizeString(filter.label).includes(normalizedQuery));
        });
        provide('filters', siftedFilters);
        /**
         * Adds the dynamic css classes to the component.
         *
         * @returns The class to be added to the component.
         * @internal
         */
        const cssClasses = computed(() => {
            return { 'x-filters-search--is-sifted': !!query.value };
        });
        /**
         * Set the query through the debounced function.
         *
         * @param query - The query to sift filters.
         * @internal
         */
        const setQuery = (query) => {
            setQueryDebounced(query);
        };
        /**
         * Clear the query.
         *
         * @internal
         */
        const clearQuery = () => {
            query.value = '';
        };
        return {
            clearQuery,
            setQuery,
            cssClasses,
            siftedFilters,
            query,
        };
    },
});

export { _sfc_main as default };
//# sourceMappingURL=filters-search.vue2.js.map
