import { isFacetFilter } from '@empathyco/x-types';
import { defineComponent } from 'vue';
import { AnimationProp } from '../../../../types/animation-prop.js';
import { toKebabCase } from '../../../../utils/string.js';
import { useFacets } from '../../composables/use-facets.js';
import { facetsXModule } from '../../x-module.js';
import _sfc_main$1 from './selected-filters.vue.js';

/**
 * This component renders a list of selected filters from every facet, or from the facet
 * ids passed as property. It uses the SelectedFilters component (state).
 *
 * It provides two slots: a scoped one which name is the filter facet id; and a default one.
 * Both exposes the filter and renders the filter label by default.
 *
 * The property "alwaysVisible" handles if the component is rendered if no filters are selected.
 *
 * @public
 */
var _sfc_main = defineComponent({
    name: 'SelectedFiltersList',
    xModule: facetsXModule.name,
    components: { SelectedFilters: _sfc_main$1 },
    props: {
        /** Array of facets ids used to get the selected filters for those facets. */
        facetsIds: Array,
        /** Flag to render the component even if there are no filters selected. */
        alwaysVisible: Boolean,
        /** Animation component that will be used to animate the selected filters list. */
        animation: {
            type: AnimationProp,
            default: 'ul',
        },
    },
    setup(props, { slots }) {
        const { selectedFilters } = useFacets(props);
        /**
         * Transforms a dictionary of Filters including the slot name.
         *
         * @param selectedFilters - A list of selected filters without slot name.
         *
         * @returns A dictionary of facets with the slot name.
         */
        function mapSlot(selectedFilters) {
            return selectedFilters.map(filter => ({
                slotName: isFacetFilter(filter) ? toKebabCase(filter.facetId) : 'default',
                selectedFilter: filter,
            }));
        }
        /**
         * Whether the slot is present in the template or not.
         *
         * @param name - The slot name.
         *
         * @returns True is the slot is present in the template. False otherwise.
         */
        function hasSlot(name) {
            return !!slots[name];
        }
        return {
            selectedFilters,
            mapSlot,
            hasSlot,
        };
    },
});

export { _sfc_main as default };
//# sourceMappingURL=selected-filters-list.vue2.js.map
