import { defineComponent, ref, computed, provide } from 'vue';
import { useFiltersInjection } from '../../composables/use-filters-injection.js';
import { facetsXModule } from '../../x-module.js';

/**
 * Component that slices a list of filters and returns them using the default scoped slot,
 * allowing the user to show the full list of them or slicing them again using the
 * show more/less buttons.
 *
 * @public
 */
var _sfc_main = defineComponent({
    name: 'SlicedFilters',
    xModule: facetsXModule.name,
    props: {
        /**
         * The list of filters to be rendered as slots.
         *
         * @public
         */
        filters: Array,
        /**
         * This prop is used in the `HierarchicalFilter` component and only in that case. It is necessary
         * to make the `renderedFilters` to return only the filters of each level of the hierarchy.
         */
        parentId: {
            type: String,
            required: false,
        },
        /** The maximum number of filters to show. */
        max: {
            type: Number,
            required: true,
        },
        buttonClass: String,
    },
    emits: ['click:show-less', 'click:show-more'],
    setup(props, { emit }) {
        /** For showing the remaining filters. */
        const showMoreFilters = ref(true);
        const renderedFilters = useFiltersInjection(props);
        /**
         * Show the buttons template when length filters is greater than property max.
         *
         * @returns Boolean if length filters is greater than property max.
         * @internal
         */
        const showButton = computed(() => renderedFilters.value.length > props.max);
        /**
         * Sliced the array of filters depends on click button show more.
         *
         * @returns Array of sliced filters or all filters.
         * @internal
         */
        const slicedFilters = computed(() => {
            return showMoreFilters.value
                ? renderedFilters.value.slice(0, props.max)
                : renderedFilters.value;
        });
        provide('filters', slicedFilters);
        /**
         * The difference between length filters and max to show.
         *
         * @returns Number of remaining filters to show.
         * @internal
         */
        const difference = computed(() => renderedFilters.value.length - props.max);
        /**
         * Show or hide the remaining filters. It also emits a Vue event based on the clicked button.
         *
         * @param event - The click event.
         *
         * @internal
         */
        const toggleShowMoreFilters = (event) => {
            showMoreFilters.value = !showMoreFilters.value;
            emit(showMoreFilters.value ? 'click:show-less' : 'click:show-more', event);
        };
        /**
         * Adds the dynamic css classes to the component.
         *
         * @returns The classes to be added to the component.
         * @internal
         */
        const cssClasses = computed(() => {
            return {
                'x-sliced-filters--is-sliced': showButton.value,
            };
        });
        return {
            cssClasses,
            toggleShowMoreFilters,
            showButton,
            difference,
            slicedFilters,
            showMoreFilters,
        };
    },
});

export { _sfc_main as default };
//# sourceMappingURL=sliced-filters.vue2.js.map
