import { defineComponent, ref, computed, onBeforeUnmount } from 'vue';
import _sfc_main$1 from '../../../components/display-emitter.vue.js';
import SlidingPanel from '../../../components/sliding-panel.vue.js';
import '../../../composables/create-use-device.js';
import { use$x } from '../../../composables/use-_x.js';
import '@vue/devtools-api';
import '../../../plugins/devtools/timeline.devtools.js';
import '@empathyco/x-utils';
import 'rxjs/operators';
import 'rxjs';
import '../../../plugins/devtools/colors.utils.js';
import '../../../plugins/x-bus.js';
import '../../../plugins/x-plugin.js';
import 'vuex';
import '@vueuse/core';
import { useState } from '../../../composables/use-state.js';
import { relatedPromptsXModule } from '../x-module.js';
import RelatedPrompt from './related-prompt.vue.js';

/**
 * This component shows the list of `RelatedPrompts` components.
 *
 * If the default slot is reimplemented in the consumer, `onSelect` function will be
 * necessary to handle the selection of the related prompt and to trigger the stagger-fade-slide animation.
 *
 * @public
 */
var _sfc_main = defineComponent({
    name: 'RelatedPromptsTagList',
    xModule: relatedPromptsXModule.name,
    components: { DisplayEmitter: _sfc_main$1, RelatedPrompt, SlidingPanel },
    props: {
        /**
         * The CSS class for the left and right button of the sliding panel.
         *
         * @public
         */
        buttonClass: String,
        /**
         * The boolean prop to handle the visiblity of sliding pannel buttons.
         *
         * @public
         */
        showButtons: { type: Boolean, default: true },
        /**
         * The CSS class for the wrapper of all the related prompt wrapper elements.
         *
         * @public
         */
        scrollContainerClass: String,
        /**
         * The CSS class for all the related prompt wrapper elements.
         *
         * @public
         */
        tagClass: String,
        /**
         * Array of colors to apply to the related prompts. It will be applied to tag
         * elements cyclically according to their index in the nex way: `tagColors[index % tagColors.length]`.
         *
         * @public
         */
        tagColors: Array,
        /**
         * The duration of the total animation in milliseconds.
         *
         * @public
         */
        animationDurationInMs: {
            type: Number,
            default: 700,
        },
    },
    setup(props) {
        const x = use$x();
        const { relatedPrompts, selectedPrompt: selectedPromptIndex } = useState('relatedPrompts');
        const clickedListItemIndex = ref(null);
        const initialOffsetLefts = {};
        const isAnimating = ref(false);
        const listItems = ref([]);
        const sortedListItems = computed(() => [...listItems.value].sort((a, b) => Number.parseInt(b.getAttribute('data-index')) -
            Number.parseInt(a.getAttribute('data-index'))));
        // The duration of a single animation (enter or leave) in milliseconds
        // if a related prompt is clicked (clickedListItemIndex.value !== null), the duration is divided by the number of related
        // prompts -1 (the clicked one is synchronized with the last one to leave or the first one to enter)
        const singleAnimationDurationInMs = computed(() => props.animationDurationInMs /
            (clickedListItemIndex.value !== null
                ? relatedPrompts.value.length - 1
                : relatedPrompts.value.length));
        const indexRelatedPrompts = computed(() => relatedPrompts.value.map((relatedPrompt, index) => ({ ...relatedPrompt, index })));
        const visibleRelatedPrompts = computed(() => selectedPromptIndex.value !== -1
            ? [indexRelatedPrompts.value[selectedPromptIndex.value]]
            : indexRelatedPrompts.value);
        let timeOutId;
        const resetTransitionStyle = (excludedProperties = ['width']) => {
            if (timeOutId) {
                clearTimeout(timeOutId);
            }
            isAnimating.value = true;
            timeOutId = +setTimeout(() => {
                isAnimating.value = false;
                clickedListItemIndex.value = null;
                sortedListItems.value.forEach(element => {
                    element.style.cssText
                        .split(';')
                        .map(rule => rule.split(':')[0]?.trim())
                        .forEach(property => {
                        if (!excludedProperties.includes(property)) {
                            element.style.removeProperty(property);
                        }
                    });
                });
            }, props.animationDurationInMs);
        };
        const onSelect = (selectedIndex) => {
            resetTransitionStyle();
            clickedListItemIndex.value = selectedIndex;
            const selected = sortedListItems.value.find(element => Number.parseInt(element.getAttribute('data-index')) === selectedIndex);
            // selectedPromptIndex.value === -1 ? 'SELECTING' : 'DESELECTING'
            if (selectedPromptIndex.value === -1) {
                // Prepare all the elements for the leave animation (~ 'beforeLeave' hook). Remember the elements are
                // sorted in descending order by index.
                sortedListItems.value.forEach(element => {
                    const index = Number.parseInt(element.getAttribute('data-index'));
                    initialOffsetLefts[index] = element.offsetLeft;
                    element.style.left = `${element.offsetLeft}px`;
                    element.style.position = 'absolute';
                    element.style.transitionDuration = `${singleAnimationDurationInMs.value}ms`;
                    if (index !== selectedIndex) {
                        element.style.opacity = '1';
                        element.style.transitionDelay = `${(index < selectedIndex ? index : index - 1) * singleAnimationDurationInMs.value}ms`;
                    }
                });
                // Synchronize the transition delay of the selected element with the last
                // element to leave
                selected.style.transitionDelay = `${(relatedPrompts.value.length > 1 ? relatedPrompts.value.length - 2 : 0) *
                    singleAnimationDurationInMs.value}ms`;
                // Trigger the animation (selecting) for the selected element
                requestAnimationFrame(() => {
                    const maxWidth = getComputedStyle(selected).maxWidth;
                    selected.style.left = '0px';
                    selected.style.setProperty('width', `${maxWidth !== 'none' ? maxWidth : '100%'}`, 'important');
                });
            }
            else {
                // Prepare the selected element for the deselecting animation
                selected.style.transitionDuration = `${singleAnimationDurationInMs.value}ms`;
                selected.style.left = '0px';
                selected.style.position = 'absolute';
                // Trigger the animation (deselecting) for the selected element
                selected.style.removeProperty('width');
                requestAnimationFrame(() => {
                    selected.style.left = `${initialOffsetLefts[selectedIndex]}px`;
                });
            }
            x.emit('UserSelectedARelatedPrompt', selectedIndex, {
                relatedPrompt: relatedPrompts.value[selectedIndex],
                selectedPrompt: selectedPromptIndex.value,
            });
        };
        const onBeforeEnter = (el) => {
            const element = el;
            const index = Number.parseInt(element.getAttribute('data-index'));
            // Prepare the element for the enter animation
            element.style.opacity = '0';
            element.style.transform = 'translateY(5px)';
            element.style.transitionDelay = `${(clickedListItemIndex.value !== null && index > clickedListItemIndex.value
                ? index - 1
                : index) * singleAnimationDurationInMs.value}ms`;
            element.style.transitionDuration = `${singleAnimationDurationInMs.value}ms`;
        };
        const onEnter = (el, done) => {
            const element = el;
            const index = Number.parseInt(element.getAttribute('data-index'));
            // Also part of the preparation for the enter animation, but it needs to be done
            // once the element is inserted in DOM (if not the offsetLeft will be always 0)
            element.style.left = `${initialOffsetLefts[index] ?? element.offsetLeft}px`;
            // trigger enter animation
            requestAnimationFrame(() => {
                element.style.opacity = '1';
                element.style.position = 'absolute';
                element.style.transform = 'translateY(0)';
            });
            done();
        };
        const onLeave = (el, done) => {
            const element = el;
            // trigger leave animation
            requestAnimationFrame(() => {
                element.style.opacity = '0';
                element.style.transform = 'translateY(5px)';
            });
            // Wait for the animation to finish (done() exectution extracts the element from the DOM)
            setTimeout(done, props.animationDurationInMs);
        };
        // Changing the request will trigger the appear animation, so we need to reset the
        // style after it finishes
        x.on('SearchRequestChanged', false).subscribe(() => {
            resetTransitionStyle([]);
        });
        onBeforeUnmount(() => {
            x.emit('RelatedPromptsUnmounted');
        });
        return {
            onSelect,
            onBeforeEnter,
            onEnter,
            onLeave,
            selectedPromptIndex,
            visibleRelatedPrompts,
            listItems,
            isAnimating,
            x,
        };
    },
});

export { _sfc_main as default };
//# sourceMappingURL=related-prompts-tag-list.vue2.js.map
