import { defineComponent, ref, computed } from 'vue';
import { use$x } from '../../../composables/use-_x.js';
import { useState } from '../../../composables/use-state.js';
import { relatedTagsXModule } from '../x-module.js';

/**
 * This component renders a related tag for a query. A related tag is a descriptive keyword
 * related to the current query to fine-tune the search. For example, if you are searching
 * for *lego*, a related tag could be *city*, refining the search with *lego city*.
 *
 * @public
 */
var _sfc_main = defineComponent({
    name: 'RelatedTag',
    xModule: relatedTagsXModule.name,
    props: {
        /**
         * Indicates if the curated related tag should be highlighted.
         *
         * @public
         */
        highlightCurated: {
            type: Boolean,
            default: false,
        },
        /**
         * The related tag model data.
         *
         * @public
         */
        relatedTag: {
            type: Object,
            required: true,
        },
    },
    setup(props) {
        const $x = use$x();
        const buttonEl = ref();
        /**
         * The selected related tags.
         *
         * @internal
         */
        const { selectedRelatedTags } = useState('relatedTags');
        /**
         * Check if the related tag is selected or not.
         *
         * @returns If the related tag is selected.
         *
         * @internal
         */
        const isSelected = computed(() => selectedRelatedTags.value.includes(props.relatedTag));
        /**
         * Blurs the related tag if it is selected.
         *
         * @public
         */
        const blurRelatedTag = () => {
            if (isSelected.value) {
                buttonEl.value?.blur();
            }
        };
        /**
         * Generates the {@link WireMetadata} object omitting the moduleName.
         *
         * @returns The {@link WireMetadata} object omitting the moduleName.
         * @internal
         */
        const createEventMetadata = () => ({
            target: buttonEl.value,
            feature: 'related_tag',
        });
        /**
         * Emits events when the button is clicked.
         *
         * @public
         */
        const emitEvents = () => {
            // We have to emit this events first to avoid the UserPickedARelatedTag wires to change the
            // isSelected value before emitting this selection events.
            $x.emit(isSelected.value ? 'UserDeselectedARelatedTag' : 'UserSelectedARelatedTag', props.relatedTag, createEventMetadata());
            $x.emit('UserPickedARelatedTag', props.relatedTag, createEventMetadata());
        };
        /**
         * Handles the click on the button.
         *
         * @public
         */
        const clickRelatedTag = () => {
            emitEvents();
            blurRelatedTag();
        };
        /**
         * Check if the related tag is curated and should be highlighted.
         *
         * @returns True if the related tag is curated and should be highlighted.
         *
         * @internal
         */
        const shouldHighlightCurated = computed(() => props.highlightCurated && (props.relatedTag.isCurated ?? false));
        /**
         * Adds the dynamic css classes to the component.
         *
         * @returns The class to be added to the component.
         *
         * @internal
         */
        const dynamicClasses = computed(() => ({
            'x-selected x-related-tag--is-selected': isSelected.value,
            'x-related-tag--is-curated': shouldHighlightCurated.value,
        }));
        return {
            buttonEl,
            dynamicClasses,
            isSelected,
            clickRelatedTag,
            shouldHighlightCurated,
        };
    },
});

export { _sfc_main as default };
//# sourceMappingURL=related-tag.vue2.js.map
