import type { StorageService } from '@empathyco/x-storage-service';
import type { Result } from '@empathyco/x-types';
import type { Store } from 'vuex';
import type { RootXStoreState } from '../../../store/index';
import type { ExternalTaggingService } from './types';
/**
 * Default implementation for the {@link ExternalTaggingService}.
 *
 * @public
 */
export declare class DefaultExternalTaggingService implements ExternalTaggingService {
    protected localStorageService: StorageService;
    protected sessionStorageService: StorageService;
    /**
     * Session id key to use as key in the storage for result clicks.
     *
     * @public
     */
    static readonly RESULT_CLICKED_ID_KEY = "add-to-cart";
    /**
     * Session id key to use as key in the storage for add to carts.
     *
     * @public
     */
    static readonly ADD_TO_CART_ID_KEY = "checkout";
    /**
     * Global instance of the {@link ExternalTaggingService}.
     */
    static instance: ExternalTaggingService;
    constructor(localStorageService?: StorageService, sessionStorageService?: StorageService);
    protected get store(): Store<RootXStoreState>;
    protected get storageKey(): string;
    protected get storageTTLMs(): number;
    /**
     * Stores in the local storage the information from the Result clicked by the user
     * in order to be able to track the add to cart later on the result's PDP.
     *
     * @param result - The result to store.
     *
     * @public
     */
    storeResultClicked(result: Result): void;
    /**
     * Stores in the local storage the information from the Result added to the cart
     * by the user in order to be able to track the checkout later on when the checkout
     * process has been completed by shopper.
     *
     * @param result - The result to store.
     *
     * @public
     */
    storeAddToCart(result: Result): void;
    /**
     * Checks if the local storage contains a result information for the given id and moves
     * the result info from the local storage to the session storage.
     *
     * @param id - The id of the result to move to the session storage.
     *
     * @public
     */
    moveToSessionStorage(id?: string): void;
    /**
     * Checks if the session storage contains a result information for a given id or the current url.
     * If exists, it tracks the add to cart and saves the add to cart information into session
     * storage.
     *
     * @param id - The id of the result to track.
     *
     * @public
     */
    trackAddToCart(id?: string): void;
    /**
     * Calculates the browser storage key for the given id.
     *
     * @param keyPrefix - The key prefix to use in the storage.
     * @param id - The id to be used for the storage key.
     *
     * @returns The complete key to be used for storage.
     *
     * @internal
     */
    protected getStorageId(keyPrefix: string, id?: string): string | null;
    /**
     * Logs a warning message in case the tracking cannot be done.
     *
     * @internal
     */
    protected showWarningMessage(): void;
    /**
     * Returns the pathname for a given url.
     *
     * @param url - The url to get the pathname from.
     *
     * @returns The pathname of the url.
     *
     * @internal
     */
    protected getPathName(url: string): string;
}
//# sourceMappingURL=external-tagging.service.d.ts.map